/*******************************************************************
    Control an LED strip using Inline Keyboard on Telegram

    By Brian Lough
    YouTube: https://www.youtube.com/brianlough
    Tindie: https://www.tindie.com/stores/brianlough/
    Twitter: https://twitter.com/witnessmenow

        Updated by Riccardo Grosso, 06/03/2024
    https://www.ethicaldiy.org
 *******************************************************************/
#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>

#include <UniversalTelegramBot.h>
// Library for interacting with the Telegram API
// Search for "Telegegram" in the Library manager and install
// The universal Telegram library
// https://github.com/witnessmenow/Universal-Arduino-Telegram-Bot


#include <ArduinoJson.h>
// Library used for parsing Json from the API responses
// NOTE: There is a breaking change in the 6.x.x version,
// install the 5.x.x version instead
// Search for "Arduino Json" in the Arduino Library manager
// https://github.com/bblanchon/ArduinoJson

//------- Replace the following! ------

char ssid[] = "inserisci il nome della tua rete WiFi";
char password[] = "inserisci la password della rete WiFi";
// Telegram BOT Token (Get from Botfather)
#define TELEGRAM_BOT_TOKEN "inserisci il tuo toked (codice alfanumerico lungo)"


//------- ---------------------- ------


// This is the Wifi client that supports HTTPS
WiFiClientSecure client;
UniversalTelegramBot bot(TELEGRAM_BOT_TOKEN, client);

#define LED_PIN 0 // Same as D4 for Wemos
int ledStatus = 0;

/* I led e i relè abitualmente si attivano con ledon e si spengono con ledoff
   Ma in alcuni apparecchi commerciali, i comandi sono invertiti
   se lanciando il programma i led/relè si accendono quando si lancia il comando di spegnimento
   e si spengono quando dovrebbero accendersi, allora commentare le due linee seguenti */

int ledon;
int ledoff;
int changeVal = 1;  




int delayBetweenChecks = 1000;
unsigned long lastTimeChecked;   //last time messages' scan has been done

unsigned long lightTimerExpires;
boolean lightTimerActive = false;

void setup() {
  Serial.begin(115200);

  if (changeVal == 1){
    ledon = 1;
    ledoff = 0;
  }    
  else{
    ledon = 0;
    ledoff = 1;  
  }

  // Set WiFi to station mode and disconnect from an AP if it was Previously connected
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);

  pinMode(0, OUTPUT);
  digitalWrite(0, ledoff);

  // attempt to connect to Wifi network:
  Serial.print("Connecting Wifi: ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  // Only required on 2.5 Beta
  client.setInsecure();

  
  // longPoll keeps the request to Telegram open for the given amount of seconds if there are no messages
  // This hugely improves response time of the bot, but is only really suitable for projects
  // where the the initial interaction comes from Telegram as the requests will block the loop for
  // the length of the long poll
  bot.longPoll = 60;
}

void handleNewMessages(int numNewMessages) {

  for (int i = 0; i < numNewMessages; i++) {

    // If the type is a "callback_query", a inline keyboard button was pressed
    if (bot.messages[i].type ==  F("callback_query")) {
      String text = bot.messages[i].text;
      String chat_id = bot.messages[i].chat_id;
      Serial.print("Call back button pressed with text: ");
      Serial.println(text);

      if (text == F("ON")) {
        digitalWrite(LED_PIN, ledon);
        ledStatus = 1;
        bot.sendMessage(chat_id, "Il led/relè è attivo", "");
      } else if (text == F("OFF")) {
        digitalWrite(LED_PIN, ledoff);
        ledStatus = 0;
        bot.sendMessage(chat_id, "Il led/relè è spento", "");
      } else if (text.startsWith("TIME")) {
        text.replace("TIME", "");
        int timeRequested = text.toInt();    
        digitalWrite(LED_PIN, ledon);
        ledStatus = 1;
        bot.sendMessage(chat_id, "Il led/relè è stato attivato a tempo", "");
        lightTimerActive = true;
        lightTimerExpires = millis() + (timeRequested * 1000 * 60);
      }
    } else {
      String chat_id = String(bot.messages[i].chat_id);
      String text = bot.messages[i].text;

     if (text == "/status")
    {


    if (ledStatus == 1)
      {
        bot.sendMessage(chat_id, "Il led/relè è attivo", "");
      }
   if (ledStatus == 0)
      {
        bot.sendMessage(chat_id, "Il led/relè è spento", "");
      } 
   if (ledStatus == 2)
      {
        bot.sendMessage(chat_id, "Il led/relè è stato spento automaticamente", "");
      }   
    }     

      if (text == F("/options")) {

        // Keyboard Json is an array of arrays.
        // The size of the main array is how many row options the keyboard has
        // The size of the sub arrays is how many coloums that row has

        // "The Text" property is what shows up in the keyboard
        // The "callback_data" property is the text that gets sent when pressed  
        
        String keyboardJson = F("[[{ \"text\" : \"ON\", \"callback_data\" : \"ON\" },{ \"text\" : \"OFF\", \"callback_data\" : \"OFF\" }],");
        keyboardJson += F("[{ \"text\" : \"15 Min.\", \"callback_data\" : \"TIME15\" }, { \"text\" : \"30 Min.\", \"callback_data\" : \"TIME30\" }, { \"text\" : \"1 ora\", \"callback_data\" : \"TIME60\" }]]");
        bot.sendMessageWithInlineKeyboard(chat_id, "Comandi disponibili", "", keyboardJson);
      }



      // When a user first uses a bot they will send a "/start" command
      // So this is a good place to let the users know what commands are available
      if (text == F("/start")) {

        bot.sendMessage(chat_id, "/options : attiva la tastiera virtuale\n", "Markdown");
        bot.sendMessage(chat_id, "/status : mostra lo stato del led/relè\n", "Markdown");
        String keyboardJson = F("[[{ \"text\" : \"ON\", \"callback_data\" : \"ON\" },{ \"text\" : \"OFF\", \"callback_data\" : \"OFF\" }],");
        keyboardJson += F("[{ \"text\" : \"15 Min.\", \"callback_data\" : \"TIME15\" }, { \"text\" : \"30 Min.\", \"callback_data\" : \"TIME30\" }, { \"text\" : \"1 ora\", \"callback_data\" : \"TIME60\" }]]");
        bot.sendMessageWithInlineKeyboard(chat_id, "Comandi disponibili", "", keyboardJson);     
      }
    }
  }
}

void loop() {
  if (millis() > lastTimeChecked + delayBetweenChecks)  {

    // getUpdates returns 1 if there is a new message from Telegram
    int numNewMessages = bot.getUpdates(bot.last_message_received + 1);

    if (numNewMessages) {
      Serial.println("got response");
      handleNewMessages(numNewMessages);
    }

    lastTimeChecked = millis();

    if (lightTimerActive && millis() > lightTimerExpires) {
      lightTimerActive = false;
      digitalWrite(LED_PIN, ledoff);
      ledStatus = 2;
    }
  }
}
